"""AFL AFL player model."""

from ...player_model import PlayerModel


def create_afl_afl_player_model(
    identifier: str,
    player_number: str,
    name: str,
) -> PlayerModel:
    """Create a player model from AFL AFL."""
    return PlayerModel(
        identifier=identifier,
        jersey=player_number,
        kicks=None,
        fumbles=None,
        fumbles_lost=None,
        field_goals=None,
        field_goals_attempted=None,
        offensive_rebounds=None,
        assists=None,
        turnovers=None,
        name=name,
    )



"""AFL AFLTables player model."""

# pylint: disable=line-too-long,duplicate-code,too-many-arguments
import datetime
import os
from urllib.parse import urlparse

import pytest_is_running
import requests_cache

from ....cache import MEMORY
from ...player_model import PlayerModel


def _create_afl_afltables_player_model(
    player_url: str, jersey: str, kicks: int | None, name: str
) -> PlayerModel:
    o = urlparse(player_url)
    last_component = o.path.split("/")[-1]
    identifier, _ = os.path.splitext(last_component)
    jersey = "".join(filter(str.isdigit, jersey))
    return PlayerModel(
        identifier=identifier,
        jersey=jersey,
        kicks=kicks,
        fumbles=None,
        fumbles_lost=None,
        field_goals=None,
        field_goals_attempted=None,
        offensive_rebounds=None,
        assists=None,
        turnovers=None,
        name=name,
    )


@MEMORY.cache
def _cached_create_afl_afltables_player_model(
    player_url: str, jersey: str, kicks: int | None, name: str
) -> PlayerModel:
    return _create_afl_afltables_player_model(player_url, jersey, kicks, name)


def create_afl_afltables_player_model(
    player_url: str,
    jersey: str,
    kicks: int | None,
    dt: datetime.datetime,
    session: requests_cache.CachedSession,
    name: str,
) -> PlayerModel:
    """Create a player model from AFL Tables."""
    if (
        not pytest_is_running.is_running()
        and dt < datetime.datetime.now() - datetime.timedelta(days=7)
    ):
        return _cached_create_afl_afltables_player_model(
            player_url,
            jersey,
            kicks,
            name,  # pyright: ignore
        )
    with session.cache_disabled():
        return _create_afl_afltables_player_model(player_url, jersey, kicks, name)



"""Tests for the player model class."""
import unittest

from sportsball.data.player_model import PlayerModel


class TestPlayerModel(unittest.TestCase):

    def setUp(self):
        self._player_model = PlayerModel(
            identifier="a",
            jersey="45",
            kicks=None,
            fumbles=None,
            fumbles_lost=None,
            field_goals=None,
            field_goals_attempted=10,
            offensive_rebounds=None,
            assists=None,
            turnovers=None,
            name="James Jones",
        )

    def test_field_goals_attempted(self):
        self.assertEqual(self._player_model.field_goals_attempted, 10)



"""Combined player model."""

# pylint: disable=too-many-locals
from ..player_model import PlayerModel


def create_combined_player_model(
    player_models: list[PlayerModel], identifier: str
) -> PlayerModel:
    """Create a player model by combining many player models."""
    jersey = None
    kicks = None
    fumbles = None
    fumbles_lost = None
    field_goals = None
    field_goals_attempted = None
    offensive_rebounds = None
    assists = None
    turnovers = None
    name = None
    for player_model in player_models:
        player_model_jersey = player_model.jersey
        if player_model_jersey is not None:
            jersey = player_model_jersey
        player_model_kicks = player_model.kicks
        if player_model_kicks is not None:
            kicks = player_model_kicks
        player_model_fumbles = player_model.fumbles
        if player_model_fumbles is not None:
            fumbles = player_model_fumbles
        player_model_fumbles_lost = player_model.fumbles_lost
        if player_model_fumbles_lost is not None:
            fumbles_lost = player_model_fumbles_lost
        player_model_field_goals = player_model.field_goals
        if player_model_field_goals is not None:
            field_goals = player_model_field_goals
        player_model_field_goals_attempted = player_model.field_goals_attempted
        if player_model_field_goals_attempted is not None:
            field_goals_attempted = player_model_field_goals_attempted
        player_model_offensive_rebounds = player_model.offensive_rebounds
        if player_model_offensive_rebounds is not None:
            offensive_rebounds = player_model_offensive_rebounds
        player_model_assists = player_model.assists
        if player_model_assists is not None:
            assists = player_model_assists
        player_model_turnovers = player_model.turnovers
        if player_model_turnovers is not None:
            turnovers = player_model_turnovers
        player_model_name = player_model.name
        if player_model_name is not None:
            name = player_model_name
    if name is None:
        raise ValueError("name is null.")
    return PlayerModel(
        identifier=identifier,
        jersey=jersey,
        kicks=kicks,
        fumbles=fumbles,
        fumbles_lost=fumbles_lost,
        field_goals=field_goals,
        field_goals_attempted=field_goals_attempted,
        offensive_rebounds=offensive_rebounds,
        assists=assists,
        turnovers=turnovers,
        name=name,
    )



"""The prototype class for a team."""

# pylint: disable=duplicate-code
from typing import Any, Literal

from pydantic import BaseModel, Field

from .field_type import TYPE_KEY, FieldType
from .news_model import NewsModel
from .odds_model import OddsModel
from .player_model import PlayerModel
from .social_model import SocialModel

TEAM_POINTS_COLUMN: Literal["points"] = "points"
TEAM_IDENTIFIER_COLUMN: Literal["identifier"] = "identifier"
PLAYER_COLUMN_PREFIX: Literal["players"] = "players"
NAME_COLUMN: Literal["name"] = "name"
FIELD_GOALS_COLUMN: Literal["field_goals"] = "field_goals"
FIELD_GOALS_ATTEMPTED_COLUMN: Literal["field_goals_attempted"] = "field_goals_attempted"
OFFENSIVE_REBOUNDS_COLUMN: Literal["offensive_rebounds"] = "offensive_rebounds"
ASSISTS_COLUMN: Literal["assists"] = "assists"
TURNOVERS_COLUMN: Literal["turnovers"] = "turnovers"
KICKS_COLUMN: Literal["kicks"] = "kicks"
TEAM_ODDS_COLUMN: Literal["odds"] = "odds"


def _calculate_kicks(data: dict[str, Any]) -> int | None:
    kicks = 0
    found_kicks = False
    for player in data[PLAYER_COLUMN_PREFIX]:
        player_kicks = player.kicks
        if player_kicks is None:
            continue
        found_kicks = True
        kicks += player_kicks
    if not found_kicks:
        return None
    return kicks


def _calculate_field_goals(data: dict[str, Any]) -> int | None:
    field_goals = 0
    found_field_goals = False
    for player in data[PLAYER_COLUMN_PREFIX]:
        player_field_goals = player.field_goals
        if player_field_goals is None:
            continue
        found_field_goals = True
        field_goals += player_field_goals
    if not found_field_goals:
        return None
    return field_goals


def _calculate_field_goals_attempted(data: dict[str, Any]) -> int | None:
    field_goals_attempted = 0
    found_field_goals_attempted = False
    for player in data[PLAYER_COLUMN_PREFIX]:
        player_field_goals_attempted = player.field_goals_attempted
        if player_field_goals_attempted is None:
            continue
        found_field_goals_attempted = True
        field_goals_attempted += player_field_goals_attempted
    if not found_field_goals_attempted:
        return None
    return field_goals_attempted


def _calculate_offensive_rebounds(data: dict[str, Any]) -> int | None:
    offensive_rebounds = 0
    found_offensive_rebounds = False
    for player in data[PLAYER_COLUMN_PREFIX]:
        player_offensive_rebounds = player.offensive_rebounds
        if player_offensive_rebounds is None:
            continue
        found_offensive_rebounds = True
        offensive_rebounds += player_offensive_rebounds
    if not found_offensive_rebounds:
        return None
    return offensive_rebounds


def _calculate_assists(data: dict[str, Any]) -> int | None:
    assists = 0
    found_assists = False
    for player in data[PLAYER_COLUMN_PREFIX]:
        player_assists = player.assists
        if player_assists is None:
            continue
        found_assists = True
        assists += player_assists
    if not found_assists:
        return None
    return assists


def _calculate_turnovers(data: dict[str, Any]) -> int | None:
    turnovers = 0
    found_turnovers = False
    for player in data[PLAYER_COLUMN_PREFIX]:
        player_turnovers = player.turnovers
        if player_turnovers is None:
            continue
        found_turnovers = True
        turnovers += player_turnovers
    if not found_turnovers:
        return None
    return turnovers


class TeamModel(BaseModel):
    """The serialisable team class."""

    identifier: str = Field(
        ...,
        json_schema_extra={TYPE_KEY: FieldType.CATEGORICAL},
        alias=TEAM_IDENTIFIER_COLUMN,
    )
    name: str = Field(
        ..., json_schema_extra={TYPE_KEY: FieldType.TEXT}, alias=NAME_COLUMN
    )
    location: str | None = Field(
        ..., json_schema_extra={TYPE_KEY: FieldType.CATEGORICAL}
    )
    players: list[PlayerModel] = Field(..., alias=PLAYER_COLUMN_PREFIX)
    odds: list[OddsModel] = Field(..., alias=TEAM_ODDS_COLUMN)
    points: float | None = Field(
        ..., json_schema_extra={TYPE_KEY: FieldType.POINTS}, alias=TEAM_POINTS_COLUMN
    )
    ladder_rank: int | None
    kicks: int | None = Field(
        default_factory=_calculate_kicks,
        json_schema_extra={TYPE_KEY: FieldType.LOOKAHEAD},
        alias=KICKS_COLUMN,
    )
    news: list[NewsModel]
    social: list[SocialModel]
    field_goals: int | None = Field(
        default_factory=_calculate_field_goals,
        json_schema_extra={TYPE_KEY: FieldType.LOOKAHEAD},
        alias=FIELD_GOALS_COLUMN,
    )
    field_goals_attempted: int | None = Field(
        default_factory=_calculate_field_goals_attempted,
        json_schema_extra={TYPE_KEY: FieldType.LOOKAHEAD},
        alias=FIELD_GOALS_ATTEMPTED_COLUMN,
    )
    offensive_rebounds: int | None = Field(
        default_factory=_calculate_offensive_rebounds,
        json_schema_extra={TYPE_KEY: FieldType.LOOKAHEAD},
        alias=OFFENSIVE_REBOUNDS_COLUMN,
    )
    assists: int | None = Field(
        default_factory=_calculate_assists,
        json_schema_extra={TYPE_KEY: FieldType.LOOKAHEAD},
        alias=ASSISTS_COLUMN,
    )
    turnovers: int | None = Field(
        default_factory=_calculate_turnovers,
        json_schema_extra={TYPE_KEY: FieldType.LOOKAHEAD},
        alias=TURNOVERS_COLUMN,
    )



"""Functions for creating a logger."""

import logging

from .args import parse_args
from .loglevel import LogLevel


def setup_logger() -> None:
    """Setup the logger to perform the required logging."""
    logging.basicConfig(
        filename="sportsball.log",  # Specify the log file name
        filemode="w",  # 'a' for append, 'w' to overwrite
        encoding="utf-8",  # Optional, specify encoding
        level=logging.INFO,  # Set the logging level
        format="%(asctime)s - %(levelname)s - %(message)s",  # Define the log message format
        datefmt="%Y-%m-%d %H:%M:%S",  # Define the date and time format
    )
    logger = logging.getLogger()
    args = parse_args()
    match args.loglevel:
        case LogLevel.DEBUG:
            logger.setLevel(logging.DEBUG)
        case LogLevel.INFO:
            logger.setLevel(logging.INFO)
        case LogLevel.WARN:
            logger.setLevel(logging.WARN)
        case LogLevel.ERROR:
            logger.setLevel(logging.ERROR)
        case _:
            raise ValueError(f"Unrecognised loglevel: {args.loglevel}")



"""AFL AFL league model."""

# pylint: disable=too-many-statements,protected-access,too-many-locals
import datetime
import re
from typing import Iterator

import requests_cache
from bs4 import BeautifulSoup
from playwright.sync_api import sync_playwright

from ....playwright import ensure_install
from ...game_model import GameModel
from ...league import League
from ...league_model import LeagueModel
from .afl_afl_game_model import create_afl_afl_game_model


class AFLAFLLeagueModel(LeagueModel):
    """AFL AFL implementation of the league model."""

    def __init__(
        self, session: requests_cache.CachedSession, position: int | None = None
    ) -> None:
        super().__init__(League.AFL, session)

    @property
    def games(self) -> Iterator[GameModel]:
        # https://www.afl.com.au/matches/team-lineups
        ensure_install()
        with sync_playwright() as p:
            browser = p.chromium.launch()
            context = browser.new_context()
            page = context.new_page()
            page.goto(
                "https://www.afl.com.au/matches/team-lineups", wait_until="networkidle"
            )
            soup = BeautifulSoup(page.content(), "html.parser")
            for div in soup.find_all(
                "div", {"class": re.compile(".*js-match-list-item.*")}
            ):
                team_names = []
                for span in div.find_all(
                    "span", {"class": re.compile(".*team-lineups__team-name.*")}
                ):
                    team_names.append(span.get_text().strip())
                teams_players: list[list[tuple[str, str, str, str]]] = [[], []]
                for div_positions_row in div.find_all(
                    "div", {"class": re.compile(".*team-lineups__positions-row.*")}
                ):
                    row_players: list[list[tuple[str, str, str, str]]] = []
                    for div_team_players in div_positions_row.find_all(
                        "div",
                        {"class": re.compile(".*team-lineups__positions-players.*")},
                    ):
                        team_players: list[tuple[str, str, str, str]] = []
                        for a in div_team_players.find_all(
                            "a", {"class": re.compile(".*js-player-profile-link.*")}
                        ):
                            player_id = "afl:" + a.get("data-player-id")
                            first_name = a.get("data-first-name")
                            second_name = a.get("data-surname")
                            player_number = None
                            for span_player_number in a.find_all(
                                "span",
                                {
                                    "class": re.compile(
                                        ".*team-lineups__player-number.*"
                                    )
                                },
                            ):
                                player_number = (
                                    span_player_number.get_text()
                                    .strip()
                                    .replace("[", "")
                                    .replace("]", "")
                                )
                            if player_number is None:
                                raise ValueError("player_number is null")
                            team_players.append(
                                (player_id, player_number, first_name, second_name)
                            )
                        row_players.append(team_players)
                    for count, row_players_list in enumerate(row_players):
                        teams_players[count].extend(row_players_list)
                dt = None
                for time in div.find_all(
                    "time", {"class": re.compile(".*match-list-alt__header-time.*")}
                ):
                    dt = datetime.datetime.fromtimestamp(
                        float(time.get("data-date")) / 1000.0
                    )
                venue_name: str | None = None
                for span in div.find_all(
                    "span", {"class": re.compile(".*match-list-alt__header-venue.*")}
                ):
                    venue_name = span.get_text().strip().replace(",", "")
                if venue_name is None:
                    raise ValueError("venue_name is null")
                if dt is None:
                    raise ValueError("dt is null")
                yield create_afl_afl_game_model(
                    team_names, teams_players, dt, venue_name, self.session
                )
